// Copyright (c) 2014 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_RENDERER_CORE_CSS_PROPERTY_NAMES_H_
#define THIRD_PARTY_BLINK_RENDERER_CORE_CSS_PROPERTY_NAMES_H_

#include <stddef.h>
#include "third_party/blink/renderer/core/core_export.h"
#include "third_party/blink/renderer/platform/wtf/assertions.h"

namespace WTF {
class AtomicString;
class String;
}

namespace blink {

enum CSSPropertyID {
    CSSPropertyInvalid = 0,
    CSSPropertyVariable = 1,
    CSSPropertyAnimationDelay = 2,
    CSSPropertyAnimationDirection = 3,
    CSSPropertyAnimationDuration = 4,
    CSSPropertyAnimationFillMode = 5,
    CSSPropertyAnimationIterationCount = 6,
    CSSPropertyAnimationName = 7,
    CSSPropertyAnimationPlayState = 8,
    CSSPropertyAnimationTimingFunction = 9,
    CSSPropertyTransitionDelay = 10,
    CSSPropertyTransitionDuration = 11,
    CSSPropertyTransitionProperty = 12,
    CSSPropertyTransitionTimingFunction = 13,
    CSSPropertyColor = 14,
    CSSPropertyDirection = 15,
    CSSPropertyFontFamily = 16,
    CSSPropertyFontFeatureSettings = 17,
    CSSPropertyFontKerning = 18,
    CSSPropertyFontSize = 19,
    CSSPropertyFontSizeAdjust = 20,
    CSSPropertyFontStretch = 21,
    CSSPropertyFontStyle = 22,
    CSSPropertyFontVariantCaps = 23,
    CSSPropertyFontVariantEastAsian = 24,
    CSSPropertyFontVariantLigatures = 25,
    CSSPropertyFontVariantNumeric = 26,
    CSSPropertyFontVariationSettings = 27,
    CSSPropertyFontWeight = 28,
    CSSPropertyTextOrientation = 29,
    CSSPropertyTextRendering = 30,
    CSSPropertyWebkitFontSmoothing = 31,
    CSSPropertyWebkitLocale = 32,
    CSSPropertyWebkitTextOrientation = 33,
    CSSPropertyWebkitWritingMode = 34,
    CSSPropertyWritingMode = 35,
    CSSPropertyZoom = 36,
    CSSPropertyAlignContent = 37,
    CSSPropertyAlignItems = 38,
    CSSPropertyAlignSelf = 39,
    CSSPropertyAlignmentBaseline = 40,
    CSSPropertyAll = 41,
    CSSPropertyBackdropFilter = 42,
    CSSPropertyBackfaceVisibility = 43,
    CSSPropertyBackgroundAttachment = 44,
    CSSPropertyBackgroundBlendMode = 45,
    CSSPropertyBackgroundClip = 46,
    CSSPropertyBackgroundColor = 47,
    CSSPropertyBackgroundImage = 48,
    CSSPropertyBackgroundOrigin = 49,
    CSSPropertyBackgroundPositionX = 50,
    CSSPropertyBackgroundPositionY = 51,
    CSSPropertyBackgroundRepeatX = 52,
    CSSPropertyBackgroundRepeatY = 53,
    CSSPropertyBackgroundSize = 54,
    CSSPropertyBaselineShift = 55,
    CSSPropertyBlockSize = 56,
    CSSPropertyBorderBlockEndColor = 57,
    CSSPropertyBorderBlockEndStyle = 58,
    CSSPropertyBorderBlockEndWidth = 59,
    CSSPropertyBorderBlockStartColor = 60,
    CSSPropertyBorderBlockStartStyle = 61,
    CSSPropertyBorderBlockStartWidth = 62,
    CSSPropertyBorderBottomColor = 63,
    CSSPropertyBorderBottomLeftRadius = 64,
    CSSPropertyBorderBottomRightRadius = 65,
    CSSPropertyBorderBottomStyle = 66,
    CSSPropertyBorderBottomWidth = 67,
    CSSPropertyBorderCollapse = 68,
    CSSPropertyBorderImageOutset = 69,
    CSSPropertyBorderImageRepeat = 70,
    CSSPropertyBorderImageSlice = 71,
    CSSPropertyBorderImageSource = 72,
    CSSPropertyBorderImageWidth = 73,
    CSSPropertyBorderInlineEndColor = 74,
    CSSPropertyBorderInlineEndStyle = 75,
    CSSPropertyBorderInlineEndWidth = 76,
    CSSPropertyBorderInlineStartColor = 77,
    CSSPropertyBorderInlineStartStyle = 78,
    CSSPropertyBorderInlineStartWidth = 79,
    CSSPropertyBorderLeftColor = 80,
    CSSPropertyBorderLeftStyle = 81,
    CSSPropertyBorderLeftWidth = 82,
    CSSPropertyBorderRightColor = 83,
    CSSPropertyBorderRightStyle = 84,
    CSSPropertyBorderRightWidth = 85,
    CSSPropertyBorderTopColor = 86,
    CSSPropertyBorderTopLeftRadius = 87,
    CSSPropertyBorderTopRightRadius = 88,
    CSSPropertyBorderTopStyle = 89,
    CSSPropertyBorderTopWidth = 90,
    CSSPropertyBottom = 91,
    CSSPropertyBoxShadow = 92,
    CSSPropertyBoxSizing = 93,
    CSSPropertyBreakAfter = 94,
    CSSPropertyBreakBefore = 95,
    CSSPropertyBreakInside = 96,
    CSSPropertyBufferedRendering = 97,
    CSSPropertyCaptionSide = 98,
    CSSPropertyCaretColor = 99,
    CSSPropertyClear = 100,
    CSSPropertyClip = 101,
    CSSPropertyClipPath = 102,
    CSSPropertyClipRule = 103,
    CSSPropertyColorInterpolation = 104,
    CSSPropertyColorInterpolationFilters = 105,
    CSSPropertyColorRendering = 106,
    CSSPropertyColumnCount = 107,
    CSSPropertyColumnFill = 108,
    CSSPropertyColumnGap = 109,
    CSSPropertyColumnRuleColor = 110,
    CSSPropertyColumnRuleStyle = 111,
    CSSPropertyColumnRuleWidth = 112,
    CSSPropertyColumnSpan = 113,
    CSSPropertyColumnWidth = 114,
    CSSPropertyContain = 115,
    CSSPropertyContent = 116,
    CSSPropertyCounterIncrement = 117,
    CSSPropertyCounterReset = 118,
    CSSPropertyCursor = 119,
    CSSPropertyCx = 120,
    CSSPropertyCy = 121,
    CSSPropertyD = 122,
    CSSPropertyDisplay = 123,
    CSSPropertyDominantBaseline = 124,
    CSSPropertyEmptyCells = 125,
    CSSPropertyFill = 126,
    CSSPropertyFillOpacity = 127,
    CSSPropertyFillRule = 128,
    CSSPropertyFilter = 129,
    CSSPropertyFlexBasis = 130,
    CSSPropertyFlexDirection = 131,
    CSSPropertyFlexGrow = 132,
    CSSPropertyFlexShrink = 133,
    CSSPropertyFlexWrap = 134,
    CSSPropertyFloat = 135,
    CSSPropertyFloodColor = 136,
    CSSPropertyFloodOpacity = 137,
    CSSPropertyFontDisplay = 138,
    CSSPropertyGridAutoColumns = 139,
    CSSPropertyGridAutoFlow = 140,
    CSSPropertyGridAutoRows = 141,
    CSSPropertyGridColumnEnd = 142,
    CSSPropertyGridColumnStart = 143,
    CSSPropertyGridRowEnd = 144,
    CSSPropertyGridRowStart = 145,
    CSSPropertyGridTemplateAreas = 146,
    CSSPropertyGridTemplateColumns = 147,
    CSSPropertyGridTemplateRows = 148,
    CSSPropertyHeight = 149,
    CSSPropertyHyphens = 150,
    CSSPropertyImageOrientation = 151,
    CSSPropertyImageRendering = 152,
    CSSPropertyInlineSize = 153,
    CSSPropertyInsetBlockEnd = 154,
    CSSPropertyInsetBlockStart = 155,
    CSSPropertyInsetInlineEnd = 156,
    CSSPropertyInsetInlineStart = 157,
    CSSPropertyIsolation = 158,
    CSSPropertyJustifyContent = 159,
    CSSPropertyJustifyItems = 160,
    CSSPropertyJustifySelf = 161,
    CSSPropertyLeft = 162,
    CSSPropertyLetterSpacing = 163,
    CSSPropertyLightingColor = 164,
    CSSPropertyLineBreak = 165,
    CSSPropertyLineHeight = 166,
    CSSPropertyLineHeightStep = 167,
    CSSPropertyListStyleImage = 168,
    CSSPropertyListStylePosition = 169,
    CSSPropertyListStyleType = 170,
    CSSPropertyMarginBlockEnd = 171,
    CSSPropertyMarginBlockStart = 172,
    CSSPropertyMarginBottom = 173,
    CSSPropertyMarginInlineEnd = 174,
    CSSPropertyMarginInlineStart = 175,
    CSSPropertyMarginLeft = 176,
    CSSPropertyMarginRight = 177,
    CSSPropertyMarginTop = 178,
    CSSPropertyMarkerEnd = 179,
    CSSPropertyMarkerMid = 180,
    CSSPropertyMarkerStart = 181,
    CSSPropertyMask = 182,
    CSSPropertyMaskSourceType = 183,
    CSSPropertyMaskType = 184,
    CSSPropertyMaxBlockSize = 185,
    CSSPropertyMaxHeight = 186,
    CSSPropertyMaxInlineSize = 187,
    CSSPropertyMaxWidth = 188,
    CSSPropertyMaxZoom = 189,
    CSSPropertyMinBlockSize = 190,
    CSSPropertyMinHeight = 191,
    CSSPropertyMinInlineSize = 192,
    CSSPropertyMinWidth = 193,
    CSSPropertyMinZoom = 194,
    CSSPropertyMixBlendMode = 195,
    CSSPropertyObjectFit = 196,
    CSSPropertyObjectPosition = 197,
    CSSPropertyOffsetAnchor = 198,
    CSSPropertyOffsetDistance = 199,
    CSSPropertyOffsetPath = 200,
    CSSPropertyOffsetPosition = 201,
    CSSPropertyOffsetRotate = 202,
    CSSPropertyOpacity = 203,
    CSSPropertyOrder = 204,
    CSSPropertyOrientation = 205,
    CSSPropertyOrphans = 206,
    CSSPropertyOutlineColor = 207,
    CSSPropertyOutlineOffset = 208,
    CSSPropertyOutlineStyle = 209,
    CSSPropertyOutlineWidth = 210,
    CSSPropertyOverflowAnchor = 211,
    CSSPropertyOverflowWrap = 212,
    CSSPropertyOverflowX = 213,
    CSSPropertyOverflowY = 214,
    CSSPropertyOverscrollBehaviorX = 215,
    CSSPropertyOverscrollBehaviorY = 216,
    CSSPropertyPaddingBlockEnd = 217,
    CSSPropertyPaddingBlockStart = 218,
    CSSPropertyPaddingBottom = 219,
    CSSPropertyPaddingInlineEnd = 220,
    CSSPropertyPaddingInlineStart = 221,
    CSSPropertyPaddingLeft = 222,
    CSSPropertyPaddingRight = 223,
    CSSPropertyPaddingTop = 224,
    CSSPropertyPage = 225,
    CSSPropertyPaintOrder = 226,
    CSSPropertyPerspective = 227,
    CSSPropertyPerspectiveOrigin = 228,
    CSSPropertyPointerEvents = 229,
    CSSPropertyPosition = 230,
    CSSPropertyQuotes = 231,
    CSSPropertyR = 232,
    CSSPropertyResize = 233,
    CSSPropertyRight = 234,
    CSSPropertyRotate = 235,
    CSSPropertyRowGap = 236,
    CSSPropertyRx = 237,
    CSSPropertyRy = 238,
    CSSPropertyScale = 239,
    CSSPropertyScrollBehavior = 240,
    CSSPropertyScrollCustomization = 241,
    CSSPropertyScrollMarginBlockEnd = 242,
    CSSPropertyScrollMarginBlockStart = 243,
    CSSPropertyScrollMarginBottom = 244,
    CSSPropertyScrollMarginInlineEnd = 245,
    CSSPropertyScrollMarginInlineStart = 246,
    CSSPropertyScrollMarginLeft = 247,
    CSSPropertyScrollMarginRight = 248,
    CSSPropertyScrollMarginTop = 249,
    CSSPropertyScrollPaddingBlockEnd = 250,
    CSSPropertyScrollPaddingBlockStart = 251,
    CSSPropertyScrollPaddingBottom = 252,
    CSSPropertyScrollPaddingInlineEnd = 253,
    CSSPropertyScrollPaddingInlineStart = 254,
    CSSPropertyScrollPaddingLeft = 255,
    CSSPropertyScrollPaddingRight = 256,
    CSSPropertyScrollPaddingTop = 257,
    CSSPropertyScrollSnapAlign = 258,
    CSSPropertyScrollSnapStop = 259,
    CSSPropertyScrollSnapType = 260,
    CSSPropertyShapeImageThreshold = 261,
    CSSPropertyShapeMargin = 262,
    CSSPropertyShapeOutside = 263,
    CSSPropertyShapeRendering = 264,
    CSSPropertySize = 265,
    CSSPropertySpeak = 266,
    CSSPropertySrc = 267,
    CSSPropertyStopColor = 268,
    CSSPropertyStopOpacity = 269,
    CSSPropertyStroke = 270,
    CSSPropertyStrokeDasharray = 271,
    CSSPropertyStrokeDashoffset = 272,
    CSSPropertyStrokeLinecap = 273,
    CSSPropertyStrokeLinejoin = 274,
    CSSPropertyStrokeMiterlimit = 275,
    CSSPropertyStrokeOpacity = 276,
    CSSPropertyStrokeWidth = 277,
    CSSPropertyTabSize = 278,
    CSSPropertyTableLayout = 279,
    CSSPropertyTextAlign = 280,
    CSSPropertyTextAlignLast = 281,
    CSSPropertyTextAnchor = 282,
    CSSPropertyTextCombineUpright = 283,
    CSSPropertyTextDecorationColor = 284,
    CSSPropertyTextDecorationLine = 285,
    CSSPropertyTextDecorationSkipInk = 286,
    CSSPropertyTextDecorationStyle = 287,
    CSSPropertyTextIndent = 288,
    CSSPropertyTextJustify = 289,
    CSSPropertyTextOverflow = 290,
    CSSPropertyTextShadow = 291,
    CSSPropertyTextSizeAdjust = 292,
    CSSPropertyTextTransform = 293,
    CSSPropertyTextUnderlinePosition = 294,
    CSSPropertyTop = 295,
    CSSPropertyTouchAction = 296,
    CSSPropertyTransform = 297,
    CSSPropertyTransformBox = 298,
    CSSPropertyTransformOrigin = 299,
    CSSPropertyTransformStyle = 300,
    CSSPropertyTranslate = 301,
    CSSPropertyUnicodeBidi = 302,
    CSSPropertyUnicodeRange = 303,
    CSSPropertyUserSelect = 304,
    CSSPropertyUserZoom = 305,
    CSSPropertyVectorEffect = 306,
    CSSPropertyVerticalAlign = 307,
    CSSPropertyViewportFit = 308,
    CSSPropertyVisibility = 309,
    CSSPropertyWebkitAppRegion = 310,
    CSSPropertyWebkitAppearance = 311,
    CSSPropertyWebkitBorderHorizontalSpacing = 312,
    CSSPropertyWebkitBorderImage = 313,
    CSSPropertyWebkitBorderVerticalSpacing = 314,
    CSSPropertyWebkitBoxAlign = 315,
    CSSPropertyWebkitBoxDecorationBreak = 316,
    CSSPropertyWebkitBoxDirection = 317,
    CSSPropertyWebkitBoxFlex = 318,
    CSSPropertyWebkitBoxOrdinalGroup = 319,
    CSSPropertyWebkitBoxOrient = 320,
    CSSPropertyWebkitBoxPack = 321,
    CSSPropertyWebkitBoxReflect = 322,
    CSSPropertyWebkitFontSizeDelta = 323,
    CSSPropertyWebkitHighlight = 324,
    CSSPropertyWebkitHyphenateCharacter = 325,
    CSSPropertyWebkitLineBreak = 326,
    CSSPropertyWebkitLineClamp = 327,
    CSSPropertyWebkitMarginAfterCollapse = 328,
    CSSPropertyWebkitMarginBeforeCollapse = 329,
    CSSPropertyWebkitMarginBottomCollapse = 330,
    CSSPropertyWebkitMarginTopCollapse = 331,
    CSSPropertyWebkitMaskBoxImageOutset = 332,
    CSSPropertyWebkitMaskBoxImageRepeat = 333,
    CSSPropertyWebkitMaskBoxImageSlice = 334,
    CSSPropertyWebkitMaskBoxImageSource = 335,
    CSSPropertyWebkitMaskBoxImageWidth = 336,
    CSSPropertyWebkitMaskClip = 337,
    CSSPropertyWebkitMaskComposite = 338,
    CSSPropertyWebkitMaskImage = 339,
    CSSPropertyWebkitMaskOrigin = 340,
    CSSPropertyWebkitMaskPositionX = 341,
    CSSPropertyWebkitMaskPositionY = 342,
    CSSPropertyWebkitMaskRepeatX = 343,
    CSSPropertyWebkitMaskRepeatY = 344,
    CSSPropertyWebkitMaskSize = 345,
    CSSPropertyWebkitPerspectiveOriginX = 346,
    CSSPropertyWebkitPerspectiveOriginY = 347,
    CSSPropertyWebkitPrintColorAdjust = 348,
    CSSPropertyWebkitRtlOrdering = 349,
    CSSPropertyWebkitRubyPosition = 350,
    CSSPropertyWebkitTapHighlightColor = 351,
    CSSPropertyWebkitTextCombine = 352,
    CSSPropertyWebkitTextDecorationsInEffect = 353,
    CSSPropertyWebkitTextEmphasisColor = 354,
    CSSPropertyWebkitTextEmphasisPosition = 355,
    CSSPropertyWebkitTextEmphasisStyle = 356,
    CSSPropertyWebkitTextFillColor = 357,
    CSSPropertyWebkitTextSecurity = 358,
    CSSPropertyWebkitTextStrokeColor = 359,
    CSSPropertyWebkitTextStrokeWidth = 360,
    CSSPropertyWebkitTransformOriginX = 361,
    CSSPropertyWebkitTransformOriginY = 362,
    CSSPropertyWebkitTransformOriginZ = 363,
    CSSPropertyWebkitUserDrag = 364,
    CSSPropertyWebkitUserModify = 365,
    CSSPropertyWhiteSpace = 366,
    CSSPropertyWidows = 367,
    CSSPropertyWidth = 368,
    CSSPropertyWillChange = 369,
    CSSPropertyWordBreak = 370,
    CSSPropertyWordSpacing = 371,
    CSSPropertyX = 372,
    CSSPropertyY = 373,
    CSSPropertyZIndex = 374,
    CSSPropertyAnimation = 375,
    CSSPropertyBackground = 376,
    CSSPropertyBackgroundPosition = 377,
    CSSPropertyBackgroundRepeat = 378,
    CSSPropertyBorder = 379,
    CSSPropertyBorderBlock = 380,
    CSSPropertyBorderBlockColor = 381,
    CSSPropertyBorderBlockEnd = 382,
    CSSPropertyBorderBlockStart = 383,
    CSSPropertyBorderBlockStyle = 384,
    CSSPropertyBorderBlockWidth = 385,
    CSSPropertyBorderBottom = 386,
    CSSPropertyBorderColor = 387,
    CSSPropertyBorderImage = 388,
    CSSPropertyBorderInline = 389,
    CSSPropertyBorderInlineColor = 390,
    CSSPropertyBorderInlineEnd = 391,
    CSSPropertyBorderInlineStart = 392,
    CSSPropertyBorderInlineStyle = 393,
    CSSPropertyBorderInlineWidth = 394,
    CSSPropertyBorderLeft = 395,
    CSSPropertyBorderRadius = 396,
    CSSPropertyBorderRight = 397,
    CSSPropertyBorderSpacing = 398,
    CSSPropertyBorderStyle = 399,
    CSSPropertyBorderTop = 400,
    CSSPropertyBorderWidth = 401,
    CSSPropertyColumnRule = 402,
    CSSPropertyColumns = 403,
    CSSPropertyFlex = 404,
    CSSPropertyFlexFlow = 405,
    CSSPropertyFont = 406,
    CSSPropertyFontVariant = 407,
    CSSPropertyGap = 408,
    CSSPropertyGrid = 409,
    CSSPropertyGridArea = 410,
    CSSPropertyGridColumn = 411,
    CSSPropertyGridColumnGap = 412,
    CSSPropertyGridGap = 413,
    CSSPropertyGridRow = 414,
    CSSPropertyGridRowGap = 415,
    CSSPropertyGridTemplate = 416,
    CSSPropertyInset = 417,
    CSSPropertyInsetBlock = 418,
    CSSPropertyInsetInline = 419,
    CSSPropertyListStyle = 420,
    CSSPropertyMargin = 421,
    CSSPropertyMarginBlock = 422,
    CSSPropertyMarginInline = 423,
    CSSPropertyMarker = 424,
    CSSPropertyOffset = 425,
    CSSPropertyOutline = 426,
    CSSPropertyOverflow = 427,
    CSSPropertyOverscrollBehavior = 428,
    CSSPropertyPadding = 429,
    CSSPropertyPaddingBlock = 430,
    CSSPropertyPaddingInline = 431,
    CSSPropertyPageBreakAfter = 432,
    CSSPropertyPageBreakBefore = 433,
    CSSPropertyPageBreakInside = 434,
    CSSPropertyPlaceContent = 435,
    CSSPropertyPlaceItems = 436,
    CSSPropertyPlaceSelf = 437,
    CSSPropertyScrollMargin = 438,
    CSSPropertyScrollMarginBlock = 439,
    CSSPropertyScrollMarginInline = 440,
    CSSPropertyScrollPadding = 441,
    CSSPropertyScrollPaddingBlock = 442,
    CSSPropertyScrollPaddingInline = 443,
    CSSPropertyTextDecoration = 444,
    CSSPropertyTransition = 445,
    CSSPropertyWebkitColumnBreakAfter = 446,
    CSSPropertyWebkitColumnBreakBefore = 447,
    CSSPropertyWebkitColumnBreakInside = 448,
    CSSPropertyWebkitMarginCollapse = 449,
    CSSPropertyWebkitMask = 450,
    CSSPropertyWebkitMaskBoxImage = 451,
    CSSPropertyWebkitMaskPosition = 452,
    CSSPropertyWebkitMaskRepeat = 453,
    CSSPropertyWebkitTextEmphasis = 454,
    CSSPropertyWebkitTextStroke = 455,
    CSSPropertyAliasWebkitBorderEndColor = 586,
    CSSPropertyAliasWebkitBorderEndStyle = 587,
    CSSPropertyAliasWebkitBorderEndWidth = 588,
    CSSPropertyAliasWebkitBorderStartColor = 589,
    CSSPropertyAliasWebkitBorderStartStyle = 590,
    CSSPropertyAliasWebkitBorderStartWidth = 591,
    CSSPropertyAliasWebkitBorderBeforeColor = 572,
    CSSPropertyAliasWebkitBorderBeforeStyle = 573,
    CSSPropertyAliasWebkitBorderBeforeWidth = 574,
    CSSPropertyAliasWebkitBorderAfterColor = 569,
    CSSPropertyAliasWebkitBorderAfterStyle = 570,
    CSSPropertyAliasWebkitBorderAfterWidth = 571,
    CSSPropertyAliasWebkitMarginEnd = 686,
    CSSPropertyAliasWebkitMarginStart = 687,
    CSSPropertyAliasWebkitMarginBefore = 684,
    CSSPropertyAliasWebkitMarginAfter = 683,
    CSSPropertyAliasWebkitPaddingEnd = 732,
    CSSPropertyAliasWebkitPaddingStart = 733,
    CSSPropertyAliasWebkitPaddingBefore = 730,
    CSSPropertyAliasWebkitPaddingAfter = 729,
    CSSPropertyAliasWebkitLogicalWidth = 665,
    CSSPropertyAliasWebkitLogicalHeight = 568,
    CSSPropertyAliasWebkitMinLogicalWidth = 704,
    CSSPropertyAliasWebkitMinLogicalHeight = 702,
    CSSPropertyAliasWebkitMaxLogicalWidth = 699,
    CSSPropertyAliasWebkitMaxLogicalHeight = 697,
    CSSPropertyAliasWebkitBorderAfter = 894,
    CSSPropertyAliasWebkitBorderBefore = 895,
    CSSPropertyAliasWebkitBorderEnd = 903,
    CSSPropertyAliasWebkitBorderStart = 904,
    CSSPropertyAliasEpubCaptionSide = 610,
    CSSPropertyAliasEpubTextCombine = 864,
    CSSPropertyAliasEpubTextEmphasis = 966,
    CSSPropertyAliasEpubTextEmphasisColor = 866,
    CSSPropertyAliasEpubTextEmphasisStyle = 868,
    CSSPropertyAliasEpubTextOrientation = 545,
    CSSPropertyAliasEpubTextTransform = 805,
    CSSPropertyAliasEpubWordBreak = 882,
    CSSPropertyAliasEpubWritingMode = 546,
    CSSPropertyAliasWebkitAlignContent = 549,
    CSSPropertyAliasWebkitAlignItems = 550,
    CSSPropertyAliasWebkitAlignSelf = 551,
    CSSPropertyAliasWebkitAnimation = 887,
    CSSPropertyAliasWebkitAnimationDelay = 514,
    CSSPropertyAliasWebkitAnimationDirection = 515,
    CSSPropertyAliasWebkitAnimationDuration = 516,
    CSSPropertyAliasWebkitAnimationFillMode = 517,
    CSSPropertyAliasWebkitAnimationIterationCount = 518,
    CSSPropertyAliasWebkitAnimationName = 519,
    CSSPropertyAliasWebkitAnimationPlayState = 520,
    CSSPropertyAliasWebkitAnimationTimingFunction = 521,
    CSSPropertyAliasWebkitBackfaceVisibility = 555,
    CSSPropertyAliasWebkitBackgroundClip = 558,
    CSSPropertyAliasWebkitBackgroundOrigin = 561,
    CSSPropertyAliasWebkitBackgroundSize = 566,
    CSSPropertyAliasWebkitBorderBottomLeftRadius = 576,
    CSSPropertyAliasWebkitBorderBottomRightRadius = 577,
    CSSPropertyAliasWebkitBorderRadius = 908,
    CSSPropertyAliasWebkitBorderTopLeftRadius = 599,
    CSSPropertyAliasWebkitBorderTopRightRadius = 600,
    CSSPropertyAliasWebkitBoxShadow = 604,
    CSSPropertyAliasWebkitBoxSizing = 605,
    CSSPropertyAliasWebkitClipPath = 614,
    CSSPropertyAliasWebkitColumnCount = 619,
    CSSPropertyAliasWebkitColumnGap = 621,
    CSSPropertyAliasWebkitColumnRule = 914,
    CSSPropertyAliasWebkitColumnRuleColor = 622,
    CSSPropertyAliasWebkitColumnRuleStyle = 623,
    CSSPropertyAliasWebkitColumnRuleWidth = 624,
    CSSPropertyAliasWebkitColumnSpan = 625,
    CSSPropertyAliasWebkitColumnWidth = 626,
    CSSPropertyAliasWebkitColumns = 915,
    CSSPropertyAliasWebkitFilter = 641,
    CSSPropertyAliasWebkitFlex = 916,
    CSSPropertyAliasWebkitFlexBasis = 642,
    CSSPropertyAliasWebkitFlexDirection = 643,
    CSSPropertyAliasWebkitFlexFlow = 917,
    CSSPropertyAliasWebkitFlexGrow = 644,
    CSSPropertyAliasWebkitFlexShrink = 645,
    CSSPropertyAliasWebkitFlexWrap = 646,
    CSSPropertyAliasWebkitFontFeatureSettings = 529,
    CSSPropertyAliasWebkitJustifyContent = 671,
    CSSPropertyAliasWebkitOpacity = 715,
    CSSPropertyAliasWebkitOrder = 716,
    CSSPropertyAliasWebkitPerspective = 739,
    CSSPropertyAliasWebkitPerspectiveOrigin = 740,
    CSSPropertyAliasWebkitShapeImageThreshold = 773,
    CSSPropertyAliasWebkitShapeMargin = 774,
    CSSPropertyAliasWebkitShapeOutside = 775,
    CSSPropertyAliasWebkitTextSizeAdjust = 804,
    CSSPropertyAliasWebkitTransform = 809,
    CSSPropertyAliasWebkitTransformOrigin = 811,
    CSSPropertyAliasWebkitTransformStyle = 812,
    CSSPropertyAliasWebkitTransition = 957,
    CSSPropertyAliasWebkitTransitionDelay = 522,
    CSSPropertyAliasWebkitTransitionDuration = 523,
    CSSPropertyAliasWebkitTransitionProperty = 524,
    CSSPropertyAliasWebkitTransitionTimingFunction = 525,
    CSSPropertyAliasWebkitUserSelect = 816,
    CSSPropertyAliasWordWrap = 724,
};

const CSSPropertyID kCSSPropertyAliasList[] = {
    static_cast<CSSPropertyID>(586), // CSSPropertyAliasWebkitBorderEndColor
    static_cast<CSSPropertyID>(587), // CSSPropertyAliasWebkitBorderEndStyle
    static_cast<CSSPropertyID>(588), // CSSPropertyAliasWebkitBorderEndWidth
    static_cast<CSSPropertyID>(589), // CSSPropertyAliasWebkitBorderStartColor
    static_cast<CSSPropertyID>(590), // CSSPropertyAliasWebkitBorderStartStyle
    static_cast<CSSPropertyID>(591), // CSSPropertyAliasWebkitBorderStartWidth
    static_cast<CSSPropertyID>(572), // CSSPropertyAliasWebkitBorderBeforeColor
    static_cast<CSSPropertyID>(573), // CSSPropertyAliasWebkitBorderBeforeStyle
    static_cast<CSSPropertyID>(574), // CSSPropertyAliasWebkitBorderBeforeWidth
    static_cast<CSSPropertyID>(569), // CSSPropertyAliasWebkitBorderAfterColor
    static_cast<CSSPropertyID>(570), // CSSPropertyAliasWebkitBorderAfterStyle
    static_cast<CSSPropertyID>(571), // CSSPropertyAliasWebkitBorderAfterWidth
    static_cast<CSSPropertyID>(686), // CSSPropertyAliasWebkitMarginEnd
    static_cast<CSSPropertyID>(687), // CSSPropertyAliasWebkitMarginStart
    static_cast<CSSPropertyID>(684), // CSSPropertyAliasWebkitMarginBefore
    static_cast<CSSPropertyID>(683), // CSSPropertyAliasWebkitMarginAfter
    static_cast<CSSPropertyID>(732), // CSSPropertyAliasWebkitPaddingEnd
    static_cast<CSSPropertyID>(733), // CSSPropertyAliasWebkitPaddingStart
    static_cast<CSSPropertyID>(730), // CSSPropertyAliasWebkitPaddingBefore
    static_cast<CSSPropertyID>(729), // CSSPropertyAliasWebkitPaddingAfter
    static_cast<CSSPropertyID>(665), // CSSPropertyAliasWebkitLogicalWidth
    static_cast<CSSPropertyID>(568), // CSSPropertyAliasWebkitLogicalHeight
    static_cast<CSSPropertyID>(704), // CSSPropertyAliasWebkitMinLogicalWidth
    static_cast<CSSPropertyID>(702), // CSSPropertyAliasWebkitMinLogicalHeight
    static_cast<CSSPropertyID>(699), // CSSPropertyAliasWebkitMaxLogicalWidth
    static_cast<CSSPropertyID>(697), // CSSPropertyAliasWebkitMaxLogicalHeight
    static_cast<CSSPropertyID>(894), // CSSPropertyAliasWebkitBorderAfter
    static_cast<CSSPropertyID>(895), // CSSPropertyAliasWebkitBorderBefore
    static_cast<CSSPropertyID>(903), // CSSPropertyAliasWebkitBorderEnd
    static_cast<CSSPropertyID>(904), // CSSPropertyAliasWebkitBorderStart
    static_cast<CSSPropertyID>(610), // CSSPropertyAliasEpubCaptionSide
    static_cast<CSSPropertyID>(864), // CSSPropertyAliasEpubTextCombine
    static_cast<CSSPropertyID>(966), // CSSPropertyAliasEpubTextEmphasis
    static_cast<CSSPropertyID>(866), // CSSPropertyAliasEpubTextEmphasisColor
    static_cast<CSSPropertyID>(868), // CSSPropertyAliasEpubTextEmphasisStyle
    static_cast<CSSPropertyID>(545), // CSSPropertyAliasEpubTextOrientation
    static_cast<CSSPropertyID>(805), // CSSPropertyAliasEpubTextTransform
    static_cast<CSSPropertyID>(882), // CSSPropertyAliasEpubWordBreak
    static_cast<CSSPropertyID>(546), // CSSPropertyAliasEpubWritingMode
    static_cast<CSSPropertyID>(549), // CSSPropertyAliasWebkitAlignContent
    static_cast<CSSPropertyID>(550), // CSSPropertyAliasWebkitAlignItems
    static_cast<CSSPropertyID>(551), // CSSPropertyAliasWebkitAlignSelf
    static_cast<CSSPropertyID>(887), // CSSPropertyAliasWebkitAnimation
    static_cast<CSSPropertyID>(514), // CSSPropertyAliasWebkitAnimationDelay
    static_cast<CSSPropertyID>(515), // CSSPropertyAliasWebkitAnimationDirection
    static_cast<CSSPropertyID>(516), // CSSPropertyAliasWebkitAnimationDuration
    static_cast<CSSPropertyID>(517), // CSSPropertyAliasWebkitAnimationFillMode
    static_cast<CSSPropertyID>(518), // CSSPropertyAliasWebkitAnimationIterationCount
    static_cast<CSSPropertyID>(519), // CSSPropertyAliasWebkitAnimationName
    static_cast<CSSPropertyID>(520), // CSSPropertyAliasWebkitAnimationPlayState
    static_cast<CSSPropertyID>(521), // CSSPropertyAliasWebkitAnimationTimingFunction
    static_cast<CSSPropertyID>(555), // CSSPropertyAliasWebkitBackfaceVisibility
    static_cast<CSSPropertyID>(558), // CSSPropertyAliasWebkitBackgroundClip
    static_cast<CSSPropertyID>(561), // CSSPropertyAliasWebkitBackgroundOrigin
    static_cast<CSSPropertyID>(566), // CSSPropertyAliasWebkitBackgroundSize
    static_cast<CSSPropertyID>(576), // CSSPropertyAliasWebkitBorderBottomLeftRadius
    static_cast<CSSPropertyID>(577), // CSSPropertyAliasWebkitBorderBottomRightRadius
    static_cast<CSSPropertyID>(908), // CSSPropertyAliasWebkitBorderRadius
    static_cast<CSSPropertyID>(599), // CSSPropertyAliasWebkitBorderTopLeftRadius
    static_cast<CSSPropertyID>(600), // CSSPropertyAliasWebkitBorderTopRightRadius
    static_cast<CSSPropertyID>(604), // CSSPropertyAliasWebkitBoxShadow
    static_cast<CSSPropertyID>(605), // CSSPropertyAliasWebkitBoxSizing
    static_cast<CSSPropertyID>(614), // CSSPropertyAliasWebkitClipPath
    static_cast<CSSPropertyID>(619), // CSSPropertyAliasWebkitColumnCount
    static_cast<CSSPropertyID>(621), // CSSPropertyAliasWebkitColumnGap
    static_cast<CSSPropertyID>(914), // CSSPropertyAliasWebkitColumnRule
    static_cast<CSSPropertyID>(622), // CSSPropertyAliasWebkitColumnRuleColor
    static_cast<CSSPropertyID>(623), // CSSPropertyAliasWebkitColumnRuleStyle
    static_cast<CSSPropertyID>(624), // CSSPropertyAliasWebkitColumnRuleWidth
    static_cast<CSSPropertyID>(625), // CSSPropertyAliasWebkitColumnSpan
    static_cast<CSSPropertyID>(626), // CSSPropertyAliasWebkitColumnWidth
    static_cast<CSSPropertyID>(915), // CSSPropertyAliasWebkitColumns
    static_cast<CSSPropertyID>(641), // CSSPropertyAliasWebkitFilter
    static_cast<CSSPropertyID>(916), // CSSPropertyAliasWebkitFlex
    static_cast<CSSPropertyID>(642), // CSSPropertyAliasWebkitFlexBasis
    static_cast<CSSPropertyID>(643), // CSSPropertyAliasWebkitFlexDirection
    static_cast<CSSPropertyID>(917), // CSSPropertyAliasWebkitFlexFlow
    static_cast<CSSPropertyID>(644), // CSSPropertyAliasWebkitFlexGrow
    static_cast<CSSPropertyID>(645), // CSSPropertyAliasWebkitFlexShrink
    static_cast<CSSPropertyID>(646), // CSSPropertyAliasWebkitFlexWrap
    static_cast<CSSPropertyID>(529), // CSSPropertyAliasWebkitFontFeatureSettings
    static_cast<CSSPropertyID>(671), // CSSPropertyAliasWebkitJustifyContent
    static_cast<CSSPropertyID>(715), // CSSPropertyAliasWebkitOpacity
    static_cast<CSSPropertyID>(716), // CSSPropertyAliasWebkitOrder
    static_cast<CSSPropertyID>(739), // CSSPropertyAliasWebkitPerspective
    static_cast<CSSPropertyID>(740), // CSSPropertyAliasWebkitPerspectiveOrigin
    static_cast<CSSPropertyID>(773), // CSSPropertyAliasWebkitShapeImageThreshold
    static_cast<CSSPropertyID>(774), // CSSPropertyAliasWebkitShapeMargin
    static_cast<CSSPropertyID>(775), // CSSPropertyAliasWebkitShapeOutside
    static_cast<CSSPropertyID>(804), // CSSPropertyAliasWebkitTextSizeAdjust
    static_cast<CSSPropertyID>(809), // CSSPropertyAliasWebkitTransform
    static_cast<CSSPropertyID>(811), // CSSPropertyAliasWebkitTransformOrigin
    static_cast<CSSPropertyID>(812), // CSSPropertyAliasWebkitTransformStyle
    static_cast<CSSPropertyID>(957), // CSSPropertyAliasWebkitTransition
    static_cast<CSSPropertyID>(522), // CSSPropertyAliasWebkitTransitionDelay
    static_cast<CSSPropertyID>(523), // CSSPropertyAliasWebkitTransitionDuration
    static_cast<CSSPropertyID>(524), // CSSPropertyAliasWebkitTransitionProperty
    static_cast<CSSPropertyID>(525), // CSSPropertyAliasWebkitTransitionTimingFunction
    static_cast<CSSPropertyID>(816), // CSSPropertyAliasWebkitUserSelect
    static_cast<CSSPropertyID>(724), // CSSPropertyAliasWordWrap
};

const CSSPropertyID firstCSSProperty = static_cast<CSSPropertyID>(2);
const int numCSSProperties = 554;
const CSSPropertyID lastCSSProperty = static_cast<CSSPropertyID>(455);
const CSSPropertyID lastUnresolvedCSSProperty = static_cast<CSSPropertyID>(966);
const int numCSSPropertyIDs = lastUnresolvedCSSProperty + 1;
const size_t maxCSSPropertyNameLength = 40;

inline bool isCSSPropertyIDWithName(int id)
{
    return id >= firstCSSProperty && id <= lastUnresolvedCSSProperty;
}

inline bool isValidCSSPropertyID(CSSPropertyID id)
{
    return id != CSSPropertyInvalid;
}

inline CSSPropertyID convertToCSSPropertyID(int value)
{
    DCHECK_GE(value, CSSPropertyInvalid);
    DCHECK_LE(value, lastCSSProperty);
    return static_cast<CSSPropertyID>(value);
}

inline CSSPropertyID resolveCSSPropertyID(CSSPropertyID id)
{
    return convertToCSSPropertyID(id & ~512);
}

inline bool isPropertyAlias(CSSPropertyID id) { return id & 512; }

CSSPropertyID unresolvedCSSPropertyID(const WTF::String&);

CSSPropertyID CORE_EXPORT cssPropertyID(const WTF::String&);

}  // namespace blink

#endif  // THIRD_PARTY_BLINK_RENDERER_CORE_CSS_PROPERTY_NAMES_H_
